static sym
const latin1_syms[] = {
	{ 0x00a0, "nobreakspace" },
	{ 0x00a1, "exclamdown" },
	{ 0x00a2, "cent" },
	{ 0x00a3, "sterling" },
	{ 0x00a4, "currency" },
	{ 0x00a5, "yen" },
	{ 0x00a6, "brokenbar" },
	{ 0x00a7, "section" },
	{ 0x00a8, "diaeresis" },
	{ 0x00a9, "copyright" },
	{ 0x00aa, "ordfeminine" },
	{ 0x00ab, "guillemotleft" },
	{ 0x00ac, "notsign" },
	{ 0x00ad, "hyphen" },
	{ 0x00ae, "registered" },
	{ 0x00af, "macron" },
	{ 0x00b0, "degree" },
	{ 0x00b1, "plusminus" },
	{ 0x00b2, "twosuperior" },
	{ 0x00b3, "threesuperior" },
	{ 0x00b4, "acute" },
	{ 0x00b5, "mu" },
	{ 0x00b6, "paragraph" },
	{ 0x00b7, "periodcentered" },
	{ 0x00b8, "cedilla" },
	{ 0x00b9, "onesuperior" },
	{ 0x00ba, "masculine" },
	{ 0x00bb, "guillemotright" },
	{ 0x00bc, "onequarter" },
	{ 0x00bd, "onehalf" },
	{ 0x00be, "threequarters" },
	{ 0x00bf, "questiondown" },
	{ 0x00c0, "Agrave" },
	{ 0x00c1, "Aacute" },
	{ 0x00c2, "Acircumflex" },
	{ 0x00c3, "Atilde" },
	{ 0x00c4, "Adiaeresis" },
	{ 0x00c5, "Aring" },
	{ 0x00c6, "AE" },
	{ 0x00c7, "Ccedilla" },
	{ 0x00c8, "Egrave" },
	{ 0x00c9, "Eacute" },
	{ 0x00ca, "Ecircumflex" },
	{ 0x00cb, "Ediaeresis" },
	{ 0x00cc, "Igrave" },
	{ 0x00cd, "Iacute" },
	{ 0x00ce, "Icircumflex" },
	{ 0x00cf, "Idiaeresis" },
	{ 0x00d0, "ETH" },
	{ 0x00d1, "Ntilde" },
	{ 0x00d2, "Ograve" },
	{ 0x00d3, "Oacute" },
	{ 0x00d4, "Ocircumflex" },
	{ 0x00d5, "Otilde" },
	{ 0x00d6, "Odiaeresis" },
	{ 0x00d7, "multiply" },
	{ 0x00d8, "Ooblique" },
	{ 0x00d9, "Ugrave" },
	{ 0x00da, "Uacute" },
	{ 0x00db, "Ucircumflex" },
	{ 0x00dc, "Udiaeresis" },
	{ 0x00dd, "Yacute" },
	{ 0x00de, "THORN" },
	{ 0x00df, "ssharp" },
	{ 0x00e0, "agrave" },
	{ 0x00e1, "aacute" },
	{ 0x00e2, "acircumflex" },
	{ 0x00e3, "atilde" },
	{ 0x00e4, "adiaeresis" },
	{ 0x00e5, "aring" },
	{ 0x00e6, "ae" },
	{ 0x00e7, "ccedilla" },
	{ 0x00e8, "egrave" },
	{ 0x00e9, "eacute" },
	{ 0x00ea, "ecircumflex" },
	{ 0x00eb, "ediaeresis" },
	{ 0x00ec, "igrave" },
	{ 0x00ed, "iacute" },
	{ 0x00ee, "icircumflex" },
	{ 0x00ef, "idiaeresis" },
	{ 0x00f0, "eth" },
	{ 0x00f1, "ntilde" },
	{ 0x00f2, "ograve" },
	{ 0x00f3, "oacute" },
	{ 0x00f4, "ocircumflex" },
	{ 0x00f5, "otilde" },
	{ 0x00f6, "odiaeresis" },
	{ 0x00f7, "division" },
	{ 0x00f8, "oslash" },
	{ 0x00f9, "ugrave" },
	{ 0x00fa, "uacute" },
	{ 0x00fb, "ucircumflex" },
	{ 0x00fc, "udiaeresis" },
	{ 0x00fd, "yacute" },
	{ 0x00fe, "thorn" },
	{ 0x00ff, "ydiaeresis" },
};
